@@ -1,7 +1,7 @@
 /*
  * gpe-conf
  *
- * Copyright (C) 2003, 2004  Florian Boor <florian.boor@kernelconcepts.de>
+ * Copyright (C) 2003 - 2005  Florian Boor <florian.boor@kernelconcepts.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -10,8 +10,6 @@
  *
  * GPE battery info and settings module.
  *
- * battery query stuff is taken from  gpe-batmon by 
- * Nils Faerber <nils@kernelconcepts.de>
  */
 
 #include <stdlib.h>
@@ -26,9 +24,6 @@
 #include <sys/ioctl.h> 
 #include <sys/time.h>
 #include <sys/types.h>
-#ifdef MACH_IPAQ
-#include <linux/h3600_ts.h>
-#endif
 
 #include <gpe/errorbox.h>
 #include <gpe/spacing.h>
@@ -43,6 +38,7 @@
 
 #define TS_DEV "/dev/touchscreen/0raw"
 #define PROC_BATTERY "/proc/asic/battery"
+#define HAL_BATTERY "/proc/hal/battery"
 #define PROC_APM "/proc/apm"
 
 /* local types and structs */
@@ -54,6 +50,13 @@ typedef struct
 }
 t_infowidgets;
 
+typedef struct {
+        unsigned int  chemistry;
+        unsigned int  status;
+        unsigned int  voltage;    /* Voltage for battery #0; unknown for battery #1 */
+        unsigned int  percentage; /* Percentage of full charge */
+        unsigned int  life;       /* Life remaining in minutes */
+}t_battery_data;
 
 /* local definitions */
 
@@ -63,7 +66,8 @@ gchar *bat_chemistries[] = {
 	"NiCd",
 	"NiMh",
 	"Lithium Ion",
-	"Lithium Polymer"
+	"Lithium Polymer",
+	"not installed"
 };
 
 
@@ -71,43 +75,92 @@ static GtkWidget *vbox;
 t_infowidgets batt_int;
 t_infowidgets batt_ext;
 
-static int TSfd;
 FILE *file_apm = NULL;
 
 /* local functions */
 
-
-void init_device()
+/* Check if we can read battery information from iPaq HAL. */ 
+static gboolean
+have_ipaq_hal(void)
 {
-	/* check if there is an iPaq touchscreen device, open it */
-	TSfd=open(TS_DEV, O_RDWR);
-	if (TSfd < 0) {
- 		g_print(_("could not open touchscreen device '%s'\n"), TS_DEV);
- 		perror("open");
-	};
+	if (access(HAL_BATTERY, R_OK))
+		return FALSE;
+	else
+		return TRUE;
+}
+
+/* 
+ * Read battery information from iPaq HAL, batteries needs to 
+ * have a minimum length of two - we have two batteries in iPaq
+ * sometimes.
+ */
+static int
+read_ipaq_batteries(t_battery_data *batteries)
+{
+	int num_batteries = 0;
+	gchar *batraw = NULL;
+	gchar **batinfo;
+	int pos = 0;
+	GError *err = NULL;
 	
+	if (!g_file_get_contents(HAL_BATTERY, &batraw, NULL, &err))
+		return 0;
+	batinfo = g_strsplit(batraw, "\n", 50);
+	g_free(batraw);
+	
+	while (batinfo[pos])
+	{
+		if (strstr(batinfo[pos], "Battery"))
+		{
+			memset(&batteries[num_batteries], 0, sizeof(t_battery_data));
+			num_batteries++;
+		}
+		else
+		{
+			sscanf(batinfo[pos], " Chemistry  : 0x%02x", 
+			       &batteries[num_batteries-1].chemistry);
+			sscanf(batinfo[pos], " Status     : 0x%x", 
+			       &batteries[num_batteries-1].status);
+			sscanf(batinfo[pos], " Voltage    : 0x%x", 
+			       &batteries[num_batteries-1].voltage);
+			sscanf(batinfo[pos], " Percentage : 0x%x",
+                   &batteries[num_batteries-1].percentage);
+			sscanf(batinfo[pos], " Life (min) : %i", 
+			       &batteries[num_batteries-1].life);
+		}
+		pos++;
+	}
+	g_strfreev(batinfo);
+	return num_batteries;
+}
+
+void 
+init_device()
+{
 	/* then check if we have apm support available */
 	if (!access(PROC_APM, R_OK))
-		file_apm = fopen(PROC_APM,"r");
+		file_apm = fopen(PROC_APM, "r");
 }
 
 
-gint update_bat_values(gpointer data)
+gint 
+update_bat_values(gpointer data)
 {
-	gchar percstr[9];
+	gchar percstr[32];
 	gchar tmp[128];
 #ifdef MACH_IPAQ
-	static struct h3600_battery battery_val;
+	t_battery_data battery_val[2];
+	int num_batteries;
 	float bperc;
 	gboolean islearning = FALSE;
 
 	parse_file(PROC_BATTERY," Is learning? %*s %s",tmp);
-	if (strstr(tmp,"yes"))
+	if (strstr(tmp, "yes"))
 		islearning = TRUE;
-#warning TODO: Switch to modern proc interface.	
-	if (ioctl(TSfd, GET_BATTERY_STATUS, &battery_val) == 0)
+	num_batteries = read_ipaq_batteries(battery_val);
+	if (num_batteries)
 	{
-		if (battery_val.battery_count < 2) {
+		if (num_batteries < 2) {
 			gtk_widget_hide(batt_ext.bar);
 			gtk_widget_hide(batt_ext.label);
 			gtk_widget_hide(batt_ext.lstate);
@@ -125,19 +178,23 @@ gint update_bat_values(gpointer data)
 			gtk_widget_show(batt_ext.llifetime);
 		}
 		
-		bperc=(float)battery_val.battery[0].percentage / (float)100;
+		bperc=(float)battery_val[0].percentage / (float)100;
 		if (bperc > 1.0)
 			bperc = 1.0;
 		
 		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (batt_int.bar),bperc);
 		toolbar_set_style (batt_int.bar,  90 - (int) (bperc*100));
-		sprintf(percstr,"%d %%",(unsigned char)battery_val.battery[0].percentage);
+		if (battery_val[0].percentage <= 100)
+			sprintf(percstr,"%d %%",(unsigned char)battery_val[0].percentage);
+		else
+			snprintf(percstr, 32, "%s",_("unknown"));
+			
 		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(batt_int.bar),percstr);
 	
-		if ((battery_val.battery[0].chemistry > 0) && (battery_val.battery[0].chemistry <= 0x05))
-			gtk_label_set_text(GTK_LABEL(batt_int.lchem),bat_chemistries[battery_val.battery[0].chemistry]);
+		if ((battery_val[0].chemistry > 0) && (battery_val[0].chemistry <= 0x05))
+			gtk_label_set_text(GTK_LABEL(batt_int.lchem),bat_chemistries[battery_val[0].chemistry]);
 	
-		switch (battery_val.battery[0].status) {
+		switch (battery_val[0].status) {
 			case H3600_BATT_STATUS_HIGH:
 				sprintf(tmp,"%s",_("Status: high"));
 				break;
@@ -172,29 +229,32 @@ gint update_bat_values(gpointer data)
 		
 		gtk_label_set_text(GTK_LABEL(batt_int.lstate),tmp);
 	
-		sprintf(tmp,"%s: %d mV",_("Voltage"),battery_val.battery[0].voltage * 5000 / 1024);
+		sprintf(tmp,"%s: %d mV",_("Voltage"),battery_val[0].voltage * 5000 / 1024);
 		gtk_label_set_text(GTK_LABEL(batt_int.lvoltage),tmp);
 	
-		if (battery_val.battery[0].life == 65535)
+		if (battery_val[0].life == 65535)
 			sprintf(tmp,"%s",_("AC connected"));
 		else
-			sprintf(tmp,"%s: %d min.",_("Lifetime"),battery_val.battery[0].life);
+			sprintf(tmp,"%s: %d min.",_("Lifetime"),battery_val[0].life);
 		
 		gtk_label_set_text(GTK_LABEL(batt_int.llifetime),tmp);
 	
-		if (battery_val.battery_count > 1) {
-			bperc=(float)battery_val.battery[1].percentage / (float)100;
+		if (num_batteries > 1) {
+			bperc=(float)battery_val[1].percentage / (float)100;
 			if (bperc > 1.0)
 				bperc = 1.0;
-			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (batt_ext.bar),bperc);
+			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (batt_ext.bar), bperc);
 			toolbar_set_style (batt_ext.bar,  90 - (int) (bperc*100));
-			sprintf(percstr,"%d %%",(unsigned char)battery_val.battery[1].percentage);
+			if (battery_val[1].percentage <= 100)
+				sprintf(percstr,"%d %%",(unsigned char)battery_val[1].percentage);
+			else
+				snprintf(percstr, 32, "%s",_("unknown"));
 			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(batt_ext.bar),percstr);
 		
-			if ((battery_val.battery[1].chemistry > 0) && (battery_val.battery[1].chemistry <= 0x05))
-				gtk_label_set_text(GTK_LABEL(batt_ext.lchem),bat_chemistries[battery_val.battery[1].chemistry]);
+			if ((battery_val[1].chemistry > 0) && (battery_val[1].chemistry <= 0x05))
+				gtk_label_set_text(GTK_LABEL(batt_ext.lchem),bat_chemistries[battery_val[1].chemistry]);
 		
-			switch (battery_val.battery[1].status) {
+			switch (battery_val[1].status) {
 			case H3600_BATT_STATUS_HIGH:
 					gtk_label_set_text(GTK_LABEL(batt_ext.lstate),_("Status: high"));
 					break;
@@ -224,13 +284,13 @@ gint update_bat_values(gpointer data)
 					break;
 			};
 	
-		sprintf(tmp,"%s: %d mV",_("Voltage"),battery_val.battery[1].voltage * 5000 / 1024);
+		sprintf(tmp,"%s: %d mV",_("Voltage"),battery_val[1].voltage * 5000 / 1024);
 		gtk_label_set_text(GTK_LABEL(batt_ext.lvoltage),tmp);
 	
-		if (battery_val.battery[1].life == 65535)
+		if (battery_val[1].life == 65535)
 			sprintf(tmp,"%s",_("AC connected"));
 		else
-			sprintf(tmp,"%s: %d min.",_("Lifetime"),battery_val.battery[1].life);
+			sprintf(tmp,"%s: %d min.",_("Lifetime"),battery_val[1].life);
 		gtk_label_set_text(GTK_LABEL(batt_ext.llifetime),tmp);
 			
 		} else {
@@ -238,11 +298,8 @@ gint update_bat_values(gpointer data)
 			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(batt_ext.bar),"");
 		
 			gtk_label_set_text(GTK_LABEL(batt_ext.lchem),_("not installed"));
-		
 			gtk_label_set_text(GTK_LABEL(batt_ext.lstate),_("Status: unknown"));
-	
 			gtk_label_set_text(GTK_LABEL(batt_ext.lvoltage),_("Voltage: unknown"));
-	
 			gtk_label_set_text(GTK_LABEL(batt_ext.llifetime),_("Lifetime: unknown"));
 		}
 		return TRUE;
@@ -355,8 +412,8 @@ gint update_bat_values(gpointer data)
 void
 Battery_Free_Objects ()
 {
-	close(TSfd);
-	fclose(file_apm);
+	if (file_apm) 
+		fclose(file_apm);
 }
 
 void
@@ -388,7 +445,7 @@ Battery_Build_Objects (void)
 	
 	init_device();
 	
-	for (i=0;i<2;i++)
+	for (i=0; i<2; i++)
 	{
 		if (i==0) 
 			fstr = g_strdup_printf ("%s%s %s",
