@@ -27,13 +27,13 @@
 #include "interface.h"
 #include <gpe/picturebutton.h>
 #include <gpe/errorbox.h>
+#include <gpe/pixmaps.h>
 
 #include "../applets.h"
 #include "../suid.h"
-#include "gpe/pixmaps.h"
-#include "gpe/render.h"
 
 static gchar *listTitles[3];
+extern gboolean set_own_password;
 
 pwlist *pwroot = NULL;
 
@@ -578,8 +578,10 @@ create_passwindow (pwlist *init, GtkWidget *parent)
  	 gtk_signal_connect (GTK_OBJECT(passwindow) , "destroy", 
 		      (GtkSignalFunc) freedata, (gpointer)self);
   else
+  {
+     set_own_password = TRUE;
  	 gtk_signal_connect (GTK_OBJECT(passwindow) , "destroy", 
 		      gtk_main_quit, NULL);
-
+  }
   return passwindow;
 }
